#include <Adafruit_NeoPixel.h>

const int soundSensorPin = A0;  // 聲音感測模組連接到A0腳位
const int numPixels = 14;       // 燈條上 LED 的數量
const int thresholdLow = 105;   // 設置聲音觸發的下閾值
const int thresholdHigh = 106;  // 設置聲音觸發的上閾值
const int pin = 12;             // WS2812 燈條連接到 D12 腳位

Adafruit_NeoPixel strip = Adafruit_NeoPixel(numPixels, pin, NEO_GRB + NEO_KHZ800);

int music = 0;
int rainbowCycleSpeed = 10;     // 彩虹循環速度，數字越小速度越快

void setup() {
  strip.begin();               // 初始化 WS2812 燈條
  strip.show();                // 確保所有 LED 都關閉
  Serial.begin(9600);         // 初始化串口通訊，便於調試
}

void loop() {
  int sensorValue = analogRead(soundSensorPin); // 讀取聲音感測模組的值
  Serial.print("Sound Level: ");
  Serial.println(sensorValue); // 打印出當前聲音感測值，用於調試

  // 檢查聲音值是否低於 thresholdLow 或高於 thresholdHigh
  if (sensorValue < thresholdLow || sensorValue > thresholdHigh) {
    music = min(music + 1, 90); // 當聲音值低於或高於閾值時增加 music 值，最大值為 90
  } else {
    music = max(music - 1, 0); // 當聲音值在閾值內時減少 music 值，最小值為 0
  }

  // 顯示調試信息
  Serial.print("Music Value: ");
  Serial.println(music);

  if (music == 90) {
    // 顯示彩虹效果
    rainbowCycle(rainbowCycleSpeed);
  } else {
    // 根據 music 的值來控制 LED 燈條
    int numLit = map(music, 0, 90, 0, numPixels); // 計算要點亮的 LED 數量

    for (int i = 0; i < numPixels; i++) {
      if (i < numLit) {
        // 根據 music 的範圍設置顏色
        if (music <= 30) {
          strip.setPixelColor(i, strip.Color(0, 255, 0)); // 設置顏色為綠色
        } else if (music <= 60) {
          strip.setPixelColor(i, strip.Color(255, 255, 0)); // 設置顏色為黃色
        } else {
          strip.setPixelColor(i, strip.Color(0, 0, 255)); // 設置顏色為藍色
        }
      } else {
        strip.setPixelColor(i, strip.Color(0, 0, 0)); // 熄滅 LED
      }
    }
    strip.show(); // 更新 LED 燈條
  }

  delay(100); // 等待100毫秒以避免過於頻繁的讀取
}

// 彩虹循環效果函數
void rainbowCycle(int wait) {
  static uint8_t startIndex = 0; // 用來控制彩虹循環的起始位置
  for (int i = 0; i < numPixels; i++) {
    int pixelHue = (i * 256 / numPixels + startIndex) & 255;
    strip.setPixelColor(i, strip.ColorHSV(pixelHue));
  }
  strip.show();
  startIndex++; // 更新起始位置以實現循環效果
  delay(wait); // 控制彩虹循環速度
}
